#-------------------------------------------------------------------------------
# 32 Point Groups
#-------------------------------------------------------------------------------
struct PointGroup <: AbstractFiniteGroup
    name::String
    multab::Matrix{Int64}
    inv::Vector{Int64}
    cls::Vector{Vector{Int64}}
    clsv::Vector{Int64}
    mult::Vector{Int64}
    operations::Vector{String}
end

function Base.display(g::PointGroup)
    println("Point group : $(name(g))")
    println("Group order : $(order(g))")
    println("Classes     : $(length(class(g)))")
end

export pointgroup
function pointgroup(i::Integer)
    multab = MultiplicationTables[i]
    ginv = GroupInverses[i]
    cls = GroupClasses[i]
    clsv = GroupClassVectors[i]
    mult = length.(cls)
    PointGroup(
        PointGroups[i], 
        multab,
        ginv,
        cls,
        clsv,
        mult,
        OperationNames[i]
    )
end
pointgroup(s::String) = pointgroup(PointGroupDict[s])

export groupindex, operation, matrix, rotation, repname
groupindex(g::PointGroup) = PointGroupDict[name(g)]
operation(g::PointGroup) = g.operations
operation(g::PointGroup, i) = g.operations[i]
matrix(g::PointGroup) = RotationMatrices[groupindex(g)]
matrix(g::PointGroup, i) = RotationMatrices[groupindex(g)][i]
function rotation(g::PointGroup)
    GI = groupindex(g)
    if HexagonalGroups[GI]
        invhex = inv(HexagonalAxes)
        [HexagonalAxes * m * invhex for m in RotationMatrices[GI]]
    else
        RotationMatrices[GI]
    end
end
function rotation(g::PointGroup, i::Integer)
    GI = groupindex(g)
    if HexagonalGroups[GI]
        mat = RotationMatrices[GI][i]
        HexagonalAxes * mat * inv(HexagonalAxes)
    else
        RotationMatrices[GI][i]
    end
end
function rotation(g::PointGroup, i::AbstractVector{<:Integer})
    GI = groupindex(g)
    if HexagonalGroups[GI]
        invhex = inv(HexagonalAxes)
        [HexagonalAxes * m * invhex for m in RotationMatrices[GI][i]]
    else
        RotationMatrices[GI][i]
    end
end
function repname(g::PointGroup, i, convention::Integer=3)
    if convention == 3
        RepresentationNames3[groupindex(g)][i]
    elseif convention == 1
        RepresentationNames[groupindex(g)][i]
    elseif convention == 2
        RepresentationNames2[groupindex(g)][i]
    end
end
#-------------------------------------------------------------------------------
# Raw Data
#-------------------------------------------------------------------------------
const PointGroups = [
    "C1" , "Ci" , "C2" , "Cs" , "C2h", "D2" , "C2v", "D2h", 
    "C4" , "S4" , "C4h", "D4" , "C4v", "D2d", "D4h", "C3" , 
    "C3i", "D3" , "C3v", "D3d", "C6" , "C3h", "C6h", "D6" , 
    "C6v", "D3h", "D6h", "T"  , "Th" , "O"  , "Td" , "Oh" 
]

const PointGroupDict = Dict(
    "C1"  => 1,  "Ci"  => 2,  "C2"  => 3,  "Cs"  => 4,  "C2h" => 5,  "D2"  => 6,  "C2v" => 7,  "D2h" => 8, 
    "C4"  => 9,  "S4"  => 10, "C4h" => 11, "D4"  => 12, "C4v" => 13, "D2d" => 14, "D4h" => 15, "C3"  => 16, 
    "C3i" => 17, "D3"  => 18, "C3v" => 19, "D3d" => 20, "C6"  => 21, "C3h" => 22, "C6h" => 23, "D6"  => 24, 
    "C6v" => 25, "D3h" => 26, "D6h" => 27, "T"   => 28, "Th"  => 29, "O"   => 30, "Td"  => 31, "Oh"  => 32
)

const MultiplicationTables = [
    [1;;],
    [1 2; 2 1],
    [1 2; 2 1],
    [1 2; 2 1],
    [1 2 3 4; 2 1 4 3; 3 4 1 2; 4 3 2 1],
    [1 2 3 4; 2 1 4 3; 3 4 1 2; 4 3 2 1],
    [1 2 3 4; 2 1 4 3; 3 4 1 2; 4 3 2 1],
    [1 2 3 4 5 6 7 8; 2 1 4 3 6 5 8 7; 3 4 1 2 7 8 5 6; 4 3 2 1 8 7 6 5; 5 6 7 8 1 2 3 4; 6 5 8 7 2 1 4 3; 7 8 5 6 3 4 1 2; 8 7 6 5 4 3 2 1],
    [1 2 3 4; 2 1 4 3; 3 4 2 1; 4 3 1 2],
    [1 2 3 4; 2 1 4 3; 3 4 2 1; 4 3 1 2],
    [1 2 3 4 5 6 7 8; 2 1 4 3 6 5 8 7; 3 4 2 1 7 8 6 5; 4 3 1 2 8 7 5 6; 5 6 7 8 1 2 3 4; 6 5 8 7 2 1 4 3; 7 8 6 5 3 4 2 1; 8 7 5 6 4 3 1 2],
    [1 2 3 4 5 6 7 8; 2 1 4 3 6 5 8 7; 3 4 2 1 8 7 5 6; 4 3 1 2 7 8 6 5; 5 6 7 8 1 2 3 4; 6 5 8 7 2 1 4 3; 7 8 6 5 4 3 1 2; 8 7 5 6 3 4 2 1],
    [1 2 3 4 5 6 7 8; 2 1 4 3 6 5 8 7; 3 4 2 1 8 7 5 6; 4 3 1 2 7 8 6 5; 5 6 7 8 1 2 3 4; 6 5 8 7 2 1 4 3; 7 8 6 5 4 3 1 2; 8 7 5 6 3 4 2 1],
    [1 2 3 4 5 6 7 8; 2 1 4 3 6 5 8 7; 3 4 2 1 8 7 5 6; 4 3 1 2 7 8 6 5; 5 6 7 8 1 2 3 4; 6 5 8 7 2 1 4 3; 7 8 6 5 4 3 1 2; 8 7 5 6 3 4 2 1],
    [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16; 2 1 4 3 6 5 8 7 10 9 12 11 14 13 16 15; 3 4 2 1 8 7 5 6 11 12 10 9 16 15 13 14; 4 3 1 2 7 8 6 5 12 11 9 10 15 16 14 13; 5 6 7 8 1 2 3 4 13 14 15 16 9 10 11 12; 6 5 8 7 2 1 4 3 14 13 16 15 10 9 12 11; 7 8 6 5 4 3 1 2 15 16 14 13 12 11 9 10; 8 7 5 6 3 4 2 1 16 15 13 14 11 12 10 9; 9 10 11 12 13 14 15 16 1 2 3 4 5 6 7 8; 10 9 12 11 14 13 16 15 2 1 4 3 6 5 8 7; 11 12 10 9 16 15 13 14 3 4 2 1 8 7 5 6; 12 11 9 10 15 16 14 13 4 3 1 2 7 8 6 5; 13 14 15 16 9 10 11 12 5 6 7 8 1 2 3 4; 14 13 16 15 10 9 12 11 6 5 8 7 2 1 4 3; 15 16 14 13 12 11 9 10 7 8 6 5 4 3 1 2; 16 15 13 14 11 12 10 9 8 7 5 6 3 4 2 1],
    [1 2 3; 2 3 1; 3 1 2],
    [1 2 3 4 5 6; 2 3 1 5 6 4; 3 1 2 6 4 5; 4 5 6 1 2 3; 5 6 4 2 3 1; 6 4 5 3 1 2],
    [1 2 3 4 5 6; 2 3 1 6 4 5; 3 1 2 5 6 4; 4 5 6 1 2 3; 5 6 4 3 1 2; 6 4 5 2 3 1],
    [1 2 3 4 5 6; 2 3 1 6 4 5; 3 1 2 5 6 4; 4 5 6 1 2 3; 5 6 4 3 1 2; 6 4 5 2 3 1],
    [1 2 3 4 5 6 7 8 9 10 11 12; 2 3 1 6 4 5 8 9 7 12 10 11; 3 1 2 5 6 4 9 7 8 11 12 10; 4 5 6 1 2 3 10 11 12 7 8 9; 5 6 4 3 1 2 11 12 10 9 7 8; 6 4 5 2 3 1 12 10 11 8 9 7; 7 8 9 10 11 12 1 2 3 4 5 6; 8 9 7 12 10 11 2 3 1 6 4 5; 9 7 8 11 12 10 3 1 2 5 6 4; 10 11 12 7 8 9 4 5 6 1 2 3; 11 12 10 9 7 8 5 6 4 3 1 2; 12 10 11 8 9 7 6 4 5 2 3 1],
    [1 2 3 4 5 6; 2 3 1 5 6 4; 3 1 2 6 4 5; 4 5 6 1 2 3; 5 6 4 2 3 1; 6 4 5 3 1 2],
    [1 2 3 4 5 6; 2 3 1 5 6 4; 3 1 2 6 4 5; 4 5 6 1 2 3; 5 6 4 2 3 1; 6 4 5 3 1 2],
    [1 2 3 4 5 6 7 8 9 10 11 12; 2 3 1 5 6 4 8 9 7 11 12 10; 3 1 2 6 4 5 9 7 8 12 10 11; 4 5 6 1 2 3 10 11 12 7 8 9; 5 6 4 2 3 1 11 12 10 8 9 7; 6 4 5 3 1 2 12 10 11 9 7 8; 7 8 9 10 11 12 1 2 3 4 5 6; 8 9 7 11 12 10 2 3 1 5 6 4; 9 7 8 12 10 11 3 1 2 6 4 5; 10 11 12 7 8 9 4 5 6 1 2 3; 11 12 10 8 9 7 5 6 4 2 3 1; 12 10 11 9 7 8 6 4 5 3 1 2],
    [1 2 3 4 5 6 7 8 9 10 11 12; 2 3 1 5 6 4 9 7 8 12 10 11; 3 1 2 6 4 5 8 9 7 11 12 10; 4 5 6 1 2 3 10 11 12 7 8 9; 5 6 4 2 3 1 12 10 11 9 7 8; 6 4 5 3 1 2 11 12 10 8 9 7; 7 8 9 10 11 12 1 2 3 4 5 6; 8 9 7 11 12 10 3 1 2 6 4 5; 9 7 8 12 10 11 2 3 1 5 6 4; 10 11 12 7 8 9 4 5 6 1 2 3; 11 12 10 8 9 7 6 4 5 3 1 2; 12 10 11 9 7 8 5 6 4 2 3 1],
    [1 2 3 4 5 6 7 8 9 10 11 12; 2 3 1 5 6 4 9 7 8 12 10 11; 3 1 2 6 4 5 8 9 7 11 12 10; 4 5 6 1 2 3 10 11 12 7 8 9; 5 6 4 2 3 1 12 10 11 9 7 8; 6 4 5 3 1 2 11 12 10 8 9 7; 7 8 9 10 11 12 1 2 3 4 5 6; 8 9 7 11 12 10 3 1 2 6 4 5; 9 7 8 12 10 11 2 3 1 5 6 4; 10 11 12 7 8 9 4 5 6 1 2 3; 11 12 10 8 9 7 6 4 5 3 1 2; 12 10 11 9 7 8 5 6 4 2 3 1],
    [1 2 3 4 5 6 7 8 9 10 11 12; 2 3 1 5 6 4 9 7 8 12 10 11; 3 1 2 6 4 5 8 9 7 11 12 10; 4 5 6 1 2 3 10 11 12 7 8 9; 5 6 4 2 3 1 12 10 11 9 7 8; 6 4 5 3 1 2 11 12 10 8 9 7; 7 8 9 10 11 12 1 2 3 4 5 6; 8 9 7 11 12 10 3 1 2 6 4 5; 9 7 8 12 10 11 2 3 1 5 6 4; 10 11 12 7 8 9 4 5 6 1 2 3; 11 12 10 8 9 7 6 4 5 3 1 2; 12 10 11 9 7 8 5 6 4 2 3 1],
    [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24; 2 3 1 5 6 4 9 7 8 12 10 11 14 15 13 17 18 16 21 19 20 24 22 23; 3 1 2 6 4 5 8 9 7 11 12 10 15 13 14 18 16 17 20 21 19 23 24 22; 4 5 6 1 2 3 10 11 12 7 8 9 16 17 18 13 14 15 22 23 24 19 20 21; 5 6 4 2 3 1 12 10 11 9 7 8 17 18 16 14 15 13 24 22 23 21 19 20; 6 4 5 3 1 2 11 12 10 8 9 7 18 16 17 15 13 14 23 24 22 20 21 19; 7 8 9 10 11 12 1 2 3 4 5 6 19 20 21 22 23 24 13 14 15 16 17 18; 8 9 7 11 12 10 3 1 2 6 4 5 20 21 19 23 24 22 15 13 14 18 16 17; 9 7 8 12 10 11 2 3 1 5 6 4 21 19 20 24 22 23 14 15 13 17 18 16; 10 11 12 7 8 9 4 5 6 1 2 3 22 23 24 19 20 21 16 17 18 13 14 15; 11 12 10 8 9 7 6 4 5 3 1 2 23 24 22 20 21 19 18 16 17 15 13 14; 12 10 11 9 7 8 5 6 4 2 3 1 24 22 23 21 19 20 17 18 16 14 15 13; 13 14 15 16 17 18 19 20 21 22 23 24 1 2 3 4 5 6 7 8 9 10 11 12; 14 15 13 17 18 16 21 19 20 24 22 23 2 3 1 5 6 4 9 7 8 12 10 11; 15 13 14 18 16 17 20 21 19 23 24 22 3 1 2 6 4 5 8 9 7 11 12 10; 16 17 18 13 14 15 22 23 24 19 20 21 4 5 6 1 2 3 10 11 12 7 8 9; 17 18 16 14 15 13 24 22 23 21 19 20 5 6 4 2 3 1 12 10 11 9 7 8; 18 16 17 15 13 14 23 24 22 20 21 19 6 4 5 3 1 2 11 12 10 8 9 7; 19 20 21 22 23 24 13 14 15 16 17 18 7 8 9 10 11 12 1 2 3 4 5 6; 20 21 19 23 24 22 15 13 14 18 16 17 8 9 7 11 12 10 3 1 2 6 4 5; 21 19 20 24 22 23 14 15 13 17 18 16 9 7 8 12 10 11 2 3 1 5 6 4; 22 23 24 19 20 21 16 17 18 13 14 15 10 11 12 7 8 9 4 5 6 1 2 3; 23 24 22 20 21 19 18 16 17 15 13 14 11 12 10 8 9 7 6 4 5 3 1 2; 24 22 23 21 19 20 17 18 16 14 15 13 12 10 11 9 7 8 5 6 4 2 3 1],
    [1 2 3 4 5 6 7 8 9 10 11 12; 2 1 4 3 7 8 5 6 12 11 10 9; 3 4 1 2 8 7 6 5 10 9 12 11; 4 3 2 1 6 5 8 7 11 12 9 10; 5 6 7 8 9 10 11 12 1 2 3 4; 6 5 8 7 11 12 9 10 4 3 2 1; 7 8 5 6 12 11 10 9 2 1 4 3; 8 7 6 5 10 9 12 11 3 4 1 2; 9 10 11 12 1 2 3 4 5 6 7 8; 10 9 12 11 3 4 1 2 8 7 6 5; 11 12 9 10 4 3 2 1 6 5 8 7; 12 11 10 9 2 1 4 3 7 8 5 6],
    [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24; 2 1 4 3 7 8 5 6 12 11 10 9 14 13 16 15 19 20 17 18 24 23 22 21; 3 4 1 2 8 7 6 5 10 9 12 11 15 16 13 14 20 19 18 17 22 21 24 23; 4 3 2 1 6 5 8 7 11 12 9 10 16 15 14 13 18 17 20 19 23 24 21 22; 5 6 7 8 9 10 11 12 1 2 3 4 17 18 19 20 21 22 23 24 13 14 15 16; 6 5 8 7 11 12 9 10 4 3 2 1 18 17 20 19 23 24 21 22 16 15 14 13; 7 8 5 6 12 11 10 9 2 1 4 3 19 20 17 18 24 23 22 21 14 13 16 15; 8 7 6 5 10 9 12 11 3 4 1 2 20 19 18 17 22 21 24 23 15 16 13 14; 9 10 11 12 1 2 3 4 5 6 7 8 21 22 23 24 13 14 15 16 17 18 19 20; 10 9 12 11 3 4 1 2 8 7 6 5 22 21 24 23 15 16 13 14 20 19 18 17; 11 12 9 10 4 3 2 1 6 5 8 7 23 24 21 22 16 15 14 13 18 17 20 19; 12 11 10 9 2 1 4 3 7 8 5 6 24 23 22 21 14 13 16 15 19 20 17 18; 13 14 15 16 17 18 19 20 21 22 23 24 1 2 3 4 5 6 7 8 9 10 11 12; 14 13 16 15 19 20 17 18 24 23 22 21 2 1 4 3 7 8 5 6 12 11 10 9; 15 16 13 14 20 19 18 17 22 21 24 23 3 4 1 2 8 7 6 5 10 9 12 11; 16 15 14 13 18 17 20 19 23 24 21 22 4 3 2 1 6 5 8 7 11 12 9 10; 17 18 19 20 21 22 23 24 13 14 15 16 5 6 7 8 9 10 11 12 1 2 3 4; 18 17 20 19 23 24 21 22 16 15 14 13 6 5 8 7 11 12 9 10 4 3 2 1; 19 20 17 18 24 23 22 21 14 13 16 15 7 8 5 6 12 11 10 9 2 1 4 3; 20 19 18 17 22 21 24 23 15 16 13 14 8 7 6 5 10 9 12 11 3 4 1 2; 21 22 23 24 13 14 15 16 17 18 19 20 9 10 11 12 1 2 3 4 5 6 7 8; 22 21 24 23 15 16 13 14 20 19 18 17 10 9 12 11 3 4 1 2 8 7 6 5; 23 24 21 22 16 15 14 13 18 17 20 19 11 12 9 10 4 3 2 1 6 5 8 7; 24 23 22 21 14 13 16 15 19 20 17 18 12 11 10 9 2 1 4 3 7 8 5 6],
    [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24; 2 1 4 3 7 8 5 6 12 11 10 9 14 13 16 15 19 20 17 18 24 23 22 21; 3 4 1 2 8 7 6 5 10 9 12 11 16 15 14 13 18 17 20 19 23 24 21 22; 4 3 2 1 6 5 8 7 11 12 9 10 15 16 13 14 20 19 18 17 22 21 24 23; 5 6 7 8 9 10 11 12 1 2 3 4 23 24 21 22 16 15 14 13 18 17 20 19; 6 5 8 7 11 12 9 10 4 3 2 1 24 23 22 21 14 13 16 15 19 20 17 18; 7 8 5 6 12 11 10 9 2 1 4 3 22 21 24 23 15 16 13 14 20 19 18 17; 8 7 6 5 10 9 12 11 3 4 1 2 21 22 23 24 13 14 15 16 17 18 19 20; 9 10 11 12 1 2 3 4 5 6 7 8 20 19 18 17 22 21 24 23 15 16 13 14; 10 9 12 11 3 4 1 2 8 7 6 5 19 20 17 18 24 23 22 21 14 13 16 15; 11 12 9 10 4 3 2 1 6 5 8 7 17 18 19 20 21 22 23 24 13 14 15 16; 12 11 10 9 2 1 4 3 7 8 5 6 18 17 20 19 23 24 21 22 16 15 14 13; 13 14 15 16 17 18 19 20 21 22 23 24 1 2 3 4 5 6 7 8 9 10 11 12; 14 13 16 15 19 20 17 18 24 23 22 21 2 1 4 3 7 8 5 6 12 11 10 9; 15 16 13 14 20 19 18 17 22 21 24 23 4 3 2 1 6 5 8 7 11 12 9 10; 16 15 14 13 18 17 20 19 23 24 21 22 3 4 1 2 8 7 6 5 10 9 12 11; 17 18 19 20 21 22 23 24 13 14 15 16 11 12 9 10 4 3 2 1 6 5 8 7; 18 17 20 19 23 24 21 22 16 15 14 13 12 11 10 9 2 1 4 3 7 8 5 6; 19 20 17 18 24 23 22 21 14 13 16 15 10 9 12 11 3 4 1 2 8 7 6 5; 20 19 18 17 22 21 24 23 15 16 13 14 9 10 11 12 1 2 3 4 5 6 7 8; 21 22 23 24 13 14 15 16 17 18 19 20 8 7 6 5 10 9 12 11 3 4 1 2; 22 21 24 23 15 16 13 14 20 19 18 17 7 8 5 6 12 11 10 9 2 1 4 3; 23 24 21 22 16 15 14 13 18 17 20 19 5 6 7 8 9 10 11 12 1 2 3 4; 24 23 22 21 14 13 16 15 19 20 17 18 6 5 8 7 11 12 9 10 4 3 2 1],
    [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24; 2 1 4 3 7 8 5 6 12 11 10 9 14 13 16 15 19 20 17 18 24 23 22 21; 3 4 1 2 8 7 6 5 10 9 12 11 16 15 14 13 18 17 20 19 23 24 21 22; 4 3 2 1 6 5 8 7 11 12 9 10 15 16 13 14 20 19 18 17 22 21 24 23; 5 6 7 8 9 10 11 12 1 2 3 4 21 22 23 24 13 14 15 16 17 18 19 20; 6 5 8 7 11 12 9 10 4 3 2 1 22 21 24 23 15 16 13 14 20 19 18 17; 7 8 5 6 12 11 10 9 2 1 4 3 24 23 22 21 14 13 16 15 19 20 17 18; 8 7 6 5 10 9 12 11 3 4 1 2 23 24 21 22 16 15 14 13 18 17 20 19; 9 10 11 12 1 2 3 4 5 6 7 8 17 18 19 20 21 22 23 24 13 14 15 16; 10 9 12 11 3 4 1 2 8 7 6 5 18 17 20 19 23 24 21 22 16 15 14 13; 11 12 9 10 4 3 2 1 6 5 8 7 20 19 18 17 22 21 24 23 15 16 13 14; 12 11 10 9 2 1 4 3 7 8 5 6 19 20 17 18 24 23 22 21 14 13 16 15; 13 14 15 16 17 18 19 20 21 22 23 24 1 2 3 4 5 6 7 8 9 10 11 12; 14 13 16 15 19 20 17 18 24 23 22 21 2 1 4 3 7 8 5 6 12 11 10 9; 15 16 13 14 20 19 18 17 22 21 24 23 4 3 2 1 6 5 8 7 11 12 9 10; 16 15 14 13 18 17 20 19 23 24 21 22 3 4 1 2 8 7 6 5 10 9 12 11; 17 18 19 20 21 22 23 24 13 14 15 16 9 10 11 12 1 2 3 4 5 6 7 8; 18 17 20 19 23 24 21 22 16 15 14 13 10 9 12 11 3 4 1 2 8 7 6 5; 19 20 17 18 24 23 22 21 14 13 16 15 12 11 10 9 2 1 4 3 7 8 5 6; 20 19 18 17 22 21 24 23 15 16 13 14 11 12 9 10 4 3 2 1 6 5 8 7; 21 22 23 24 13 14 15 16 17 18 19 20 5 6 7 8 9 10 11 12 1 2 3 4; 22 21 24 23 15 16 13 14 20 19 18 17 6 5 8 7 11 12 9 10 4 3 2 1; 23 24 21 22 16 15 14 13 18 17 20 19 8 7 6 5 10 9 12 11 3 4 1 2; 24 23 22 21 14 13 16 15 19 20 17 18 7 8 5 6 12 11 10 9 2 1 4 3],
    [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48; 2 1 4 3 7 8 5 6 12 11 10 9 14 13 16 15 19 20 17 18 24 23 22 21 26 25 28 27 31 32 29 30 36 35 34 33 38 37 40 39 43 44 41 42 48 47 46 45; 3 4 1 2 8 7 6 5 10 9 12 11 16 15 14 13 18 17 20 19 23 24 21 22 27 28 25 26 32 31 30 29 34 33 36 35 40 39 38 37 42 41 44 43 47 48 45 46; 4 3 2 1 6 5 8 7 11 12 9 10 15 16 13 14 20 19 18 17 22 21 24 23 28 27 26 25 30 29 32 31 35 36 33 34 39 40 37 38 44 43 42 41 46 45 48 47; 5 6 7 8 9 10 11 12 1 2 3 4 23 24 21 22 16 15 14 13 18 17 20 19 29 30 31 32 33 34 35 36 25 26 27 28 47 48 45 46 40 39 38 37 42 41 44 43; 6 5 8 7 11 12 9 10 4 3 2 1 24 23 22 21 14 13 16 15 19 20 17 18 30 29 32 31 35 36 33 34 28 27 26 25 48 47 46 45 38 37 40 39 43 44 41 42; 7 8 5 6 12 11 10 9 2 1 4 3 22 21 24 23 15 16 13 14 20 19 18 17 31 32 29 30 36 35 34 33 26 25 28 27 46 45 48 47 39 40 37 38 44 43 42 41; 8 7 6 5 10 9 12 11 3 4 1 2 21 22 23 24 13 14 15 16 17 18 19 20 32 31 30 29 34 33 36 35 27 28 25 26 45 46 47 48 37 38 39 40 41 42 43 44; 9 10 11 12 1 2 3 4 5 6 7 8 20 19 18 17 22 21 24 23 15 16 13 14 33 34 35 36 25 26 27 28 29 30 31 32 44 43 42 41 46 45 48 47 39 40 37 38; 10 9 12 11 3 4 1 2 8 7 6 5 19 20 17 18 24 23 22 21 14 13 16 15 34 33 36 35 27 28 25 26 32 31 30 29 43 44 41 42 48 47 46 45 38 37 40 39; 11 12 9 10 4 3 2 1 6 5 8 7 17 18 19 20 21 22 23 24 13 14 15 16 35 36 33 34 28 27 26 25 30 29 32 31 41 42 43 44 45 46 47 48 37 38 39 40; 12 11 10 9 2 1 4 3 7 8 5 6 18 17 20 19 23 24 21 22 16 15 14 13 36 35 34 33 26 25 28 27 31 32 29 30 42 41 44 43 47 48 45 46 40 39 38 37; 13 14 15 16 17 18 19 20 21 22 23 24 1 2 3 4 5 6 7 8 9 10 11 12 37 38 39 40 41 42 43 44 45 46 47 48 25 26 27 28 29 30 31 32 33 34 35 36; 14 13 16 15 19 20 17 18 24 23 22 21 2 1 4 3 7 8 5 6 12 11 10 9 38 37 40 39 43 44 41 42 48 47 46 45 26 25 28 27 31 32 29 30 36 35 34 33; 15 16 13 14 20 19 18 17 22 21 24 23 4 3 2 1 6 5 8 7 11 12 9 10 39 40 37 38 44 43 42 41 46 45 48 47 28 27 26 25 30 29 32 31 35 36 33 34; 16 15 14 13 18 17 20 19 23 24 21 22 3 4 1 2 8 7 6 5 10 9 12 11 40 39 38 37 42 41 44 43 47 48 45 46 27 28 25 26 32 31 30 29 34 33 36 35; 17 18 19 20 21 22 23 24 13 14 15 16 11 12 9 10 4 3 2 1 6 5 8 7 41 42 43 44 45 46 47 48 37 38 39 40 35 36 33 34 28 27 26 25 30 29 32 31; 18 17 20 19 23 24 21 22 16 15 14 13 12 11 10 9 2 1 4 3 7 8 5 6 42 41 44 43 47 48 45 46 40 39 38 37 36 35 34 33 26 25 28 27 31 32 29 30; 19 20 17 18 24 23 22 21 14 13 16 15 10 9 12 11 3 4 1 2 8 7 6 5 43 44 41 42 48 47 46 45 38 37 40 39 34 33 36 35 27 28 25 26 32 31 30 29; 20 19 18 17 22 21 24 23 15 16 13 14 9 10 11 12 1 2 3 4 5 6 7 8 44 43 42 41 46 45 48 47 39 40 37 38 33 34 35 36 25 26 27 28 29 30 31 32; 21 22 23 24 13 14 15 16 17 18 19 20 8 7 6 5 10 9 12 11 3 4 1 2 45 46 47 48 37 38 39 40 41 42 43 44 32 31 30 29 34 33 36 35 27 28 25 26; 22 21 24 23 15 16 13 14 20 19 18 17 7 8 5 6 12 11 10 9 2 1 4 3 46 45 48 47 39 40 37 38 44 43 42 41 31 32 29 30 36 35 34 33 26 25 28 27; 23 24 21 22 16 15 14 13 18 17 20 19 5 6 7 8 9 10 11 12 1 2 3 4 47 48 45 46 40 39 38 37 42 41 44 43 29 30 31 32 33 34 35 36 25 26 27 28; 24 23 22 21 14 13 16 15 19 20 17 18 6 5 8 7 11 12 9 10 4 3 2 1 48 47 46 45 38 37 40 39 43 44 41 42 30 29 32 31 35 36 33 34 28 27 26 25; 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24; 26 25 28 27 31 32 29 30 36 35 34 33 38 37 40 39 43 44 41 42 48 47 46 45 2 1 4 3 7 8 5 6 12 11 10 9 14 13 16 15 19 20 17 18 24 23 22 21; 27 28 25 26 32 31 30 29 34 33 36 35 40 39 38 37 42 41 44 43 47 48 45 46 3 4 1 2 8 7 6 5 10 9 12 11 16 15 14 13 18 17 20 19 23 24 21 22; 28 27 26 25 30 29 32 31 35 36 33 34 39 40 37 38 44 43 42 41 46 45 48 47 4 3 2 1 6 5 8 7 11 12 9 10 15 16 13 14 20 19 18 17 22 21 24 23; 29 30 31 32 33 34 35 36 25 26 27 28 47 48 45 46 40 39 38 37 42 41 44 43 5 6 7 8 9 10 11 12 1 2 3 4 23 24 21 22 16 15 14 13 18 17 20 19; 30 29 32 31 35 36 33 34 28 27 26 25 48 47 46 45 38 37 40 39 43 44 41 42 6 5 8 7 11 12 9 10 4 3 2 1 24 23 22 21 14 13 16 15 19 20 17 18; 31 32 29 30 36 35 34 33 26 25 28 27 46 45 48 47 39 40 37 38 44 43 42 41 7 8 5 6 12 11 10 9 2 1 4 3 22 21 24 23 15 16 13 14 20 19 18 17; 32 31 30 29 34 33 36 35 27 28 25 26 45 46 47 48 37 38 39 40 41 42 43 44 8 7 6 5 10 9 12 11 3 4 1 2 21 22 23 24 13 14 15 16 17 18 19 20; 33 34 35 36 25 26 27 28 29 30 31 32 44 43 42 41 46 45 48 47 39 40 37 38 9 10 11 12 1 2 3 4 5 6 7 8 20 19 18 17 22 21 24 23 15 16 13 14; 34 33 36 35 27 28 25 26 32 31 30 29 43 44 41 42 48 47 46 45 38 37 40 39 10 9 12 11 3 4 1 2 8 7 6 5 19 20 17 18 24 23 22 21 14 13 16 15; 35 36 33 34 28 27 26 25 30 29 32 31 41 42 43 44 45 46 47 48 37 38 39 40 11 12 9 10 4 3 2 1 6 5 8 7 17 18 19 20 21 22 23 24 13 14 15 16; 36 35 34 33 26 25 28 27 31 32 29 30 42 41 44 43 47 48 45 46 40 39 38 37 12 11 10 9 2 1 4 3 7 8 5 6 18 17 20 19 23 24 21 22 16 15 14 13; 37 38 39 40 41 42 43 44 45 46 47 48 25 26 27 28 29 30 31 32 33 34 35 36 13 14 15 16 17 18 19 20 21 22 23 24 1 2 3 4 5 6 7 8 9 10 11 12; 38 37 40 39 43 44 41 42 48 47 46 45 26 25 28 27 31 32 29 30 36 35 34 33 14 13 16 15 19 20 17 18 24 23 22 21 2 1 4 3 7 8 5 6 12 11 10 9; 39 40 37 38 44 43 42 41 46 45 48 47 28 27 26 25 30 29 32 31 35 36 33 34 15 16 13 14 20 19 18 17 22 21 24 23 4 3 2 1 6 5 8 7 11 12 9 10; 40 39 38 37 42 41 44 43 47 48 45 46 27 28 25 26 32 31 30 29 34 33 36 35 16 15 14 13 18 17 20 19 23 24 21 22 3 4 1 2 8 7 6 5 10 9 12 11; 41 42 43 44 45 46 47 48 37 38 39 40 35 36 33 34 28 27 26 25 30 29 32 31 17 18 19 20 21 22 23 24 13 14 15 16 11 12 9 10 4 3 2 1 6 5 8 7; 42 41 44 43 47 48 45 46 40 39 38 37 36 35 34 33 26 25 28 27 31 32 29 30 18 17 20 19 23 24 21 22 16 15 14 13 12 11 10 9 2 1 4 3 7 8 5 6; 43 44 41 42 48 47 46 45 38 37 40 39 34 33 36 35 27 28 25 26 32 31 30 29 19 20 17 18 24 23 22 21 14 13 16 15 10 9 12 11 3 4 1 2 8 7 6 5; 44 43 42 41 46 45 48 47 39 40 37 38 33 34 35 36 25 26 27 28 29 30 31 32 20 19 18 17 22 21 24 23 15 16 13 14 9 10 11 12 1 2 3 4 5 6 7 8; 45 46 47 48 37 38 39 40 41 42 43 44 32 31 30 29 34 33 36 35 27 28 25 26 21 22 23 24 13 14 15 16 17 18 19 20 8 7 6 5 10 9 12 11 3 4 1 2; 46 45 48 47 39 40 37 38 44 43 42 41 31 32 29 30 36 35 34 33 26 25 28 27 22 21 24 23 15 16 13 14 20 19 18 17 7 8 5 6 12 11 10 9 2 1 4 3; 47 48 45 46 40 39 38 37 42 41 44 43 29 30 31 32 33 34 35 36 25 26 27 28 23 24 21 22 16 15 14 13 18 17 20 19 5 6 7 8 9 10 11 12 1 2 3 4; 48 47 46 45 38 37 40 39 43 44 41 42 30 29 32 31 35 36 33 34 28 27 26 25 24 23 22 21 14 13 16 15 19 20 17 18 6 5 8 7 11 12 9 10 4 3 2 1]
]

const GroupInverses = [
    [1],
    [1, 2],
    [1, 2],
    [1, 2],
    [1, 2, 3, 4],
    [1, 2, 3, 4],
    [1, 2, 3, 4],
    [1, 2, 3, 4, 5, 6, 7, 8],
    [1, 2, 4, 3],
    [1, 2, 4, 3],
    [1, 2, 4, 3, 5, 6, 8, 7],
    [1, 2, 4, 3, 5, 6, 7, 8],
    [1, 2, 4, 3, 5, 6, 7, 8],
    [1, 2, 4, 3, 5, 6, 7, 8],
    [1, 2, 4, 3, 5, 6, 7, 8, 9, 10, 12, 11, 13, 14, 15, 16],
    [1, 3, 2],
    [1, 3, 2, 4, 6, 5],
    [1, 3, 2, 4, 5, 6],
    [1, 3, 2, 4, 5, 6],
    [1, 3, 2, 4, 5, 6, 7, 9, 8, 10, 11, 12],
    [1, 3, 2, 4, 6, 5],
    [1, 3, 2, 4, 6, 5],
    [1, 3, 2, 4, 6, 5, 7, 9, 8, 10, 12, 11],
    [1, 3, 2, 4, 6, 5, 7, 8, 9, 10, 11, 12],
    [1, 3, 2, 4, 6, 5, 7, 8, 9, 10, 11, 12],
    [1, 3, 2, 4, 6, 5, 7, 8, 9, 10, 11, 12],
    [1, 3, 2, 4, 6, 5, 7, 8, 9, 10, 11, 12, 13, 15, 14, 16, 18, 17, 19, 20, 21, 22, 23, 24],
    [1, 2, 3, 4, 9, 12, 10, 11, 5, 7, 8, 6],
    [1, 2, 3, 4, 9, 12, 10, 11, 5, 7, 8, 6, 13, 14, 15, 16, 21, 24, 22, 23, 17, 19, 20, 18],
    [1, 2, 3, 4, 9, 12, 10, 11, 5, 7, 8, 6, 13, 14, 16, 15, 20, 18, 19, 17, 23, 22, 21, 24],
    [1, 2, 3, 4, 9, 12, 10, 11, 5, 7, 8, 6, 13, 14, 16, 15, 17, 19, 18, 20, 21, 24, 23, 22],
    [1, 2, 3, 4, 9, 12, 10, 11, 5, 7, 8, 6, 13, 14, 16, 15, 20, 18, 19, 17, 23, 22, 21, 24, 25, 26, 27, 28, 33, 36, 34, 35, 29, 31, 32, 30, 37, 38, 40, 39, 44, 42, 43, 41, 47, 46, 45, 48]
]

const GroupClasses = [
    [[1]],
    [[1], [2]],
    [[1], [2]],
    [[1], [2]],
    [[1], [2], [3], [4]],
    [[1], [2], [3], [4]],
    [[1], [2], [3], [4]],
    [[1], [2], [3], [4], [5], [6], [7], [8]],
    [[1], [2], [3], [4]],
    [[1], [2], [3], [4]],
    [[1], [2], [3], [4], [5], [6], [7], [8]],
    [[1], [2], [3, 4], [5, 6], [7, 8]],
    [[1], [2], [3, 4], [5, 6], [7, 8]],
    [[1], [2], [3, 4], [5, 6], [7, 8]],
    [[1], [2], [3, 4], [5, 6], [7, 8], [9], [10], [11, 12], [13, 14], [15, 16]],
    [[1], [2], [3]],
    [[1], [2], [3], [4], [5], [6]],
    [[1], [2, 3], [4, 5, 6]],
    [[1], [2, 3], [4, 5, 6]],
    [[1], [2, 3], [4, 5, 6], [7], [8, 9], [10, 11, 12]],
    [[1], [2], [3], [4], [5], [6]],
    [[1], [2], [3], [4], [5], [6]],
    [[1], [2], [3], [4], [5], [6], [7], [8], [9], [10], [11], [12]],
    [[1], [2, 3], [4], [5, 6], [7, 8, 9], [10, 11, 12]],
    [[1], [2, 3], [4], [5, 6], [7, 8, 9], [10, 11, 12]],
    [[1], [2, 3], [4], [5, 6], [7, 8, 9], [10, 11, 12]],
    [[1], [2, 3], [4], [5, 6], [7, 8, 9], [10, 11, 12], [13], [14, 15], [16], [17, 18], [19, 20, 21], [22, 23, 24]],
    [[1], [2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12]],
    [[1], [2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12], [13], [14, 15, 16], [17, 18, 19, 20], [21, 22, 23, 24]],
    [[1], [2, 3, 4], [5, 6, 7, 8, 9, 10, 11, 12], [13, 14, 18, 19, 22, 24], [15, 16, 17, 20, 21, 23]],
    [[1], [2, 3, 4], [5, 6, 7, 8, 9, 10, 11, 12], [13, 14, 17, 20, 21, 23], [15, 16, 18, 19, 22, 24]],
    [[1], [2, 3, 4], [5, 6, 7, 8, 9, 10, 11, 12], [13, 14, 18, 19, 22, 24], [15, 16, 17, 20, 21, 23], [25], [26, 27, 28], [29, 30, 31, 32, 33, 34, 35, 36], [37, 38, 42, 43, 46, 48], [39, 40, 41, 44, 45, 47]]
]

const GroupClassVectors = [
    [1],
    [1, 2],
    [1, 2],
    [1, 2],
    [1, 2, 3, 4],
    [1, 2, 3, 4],
    [1, 2, 3, 4],
    [1, 2, 3, 4, 5, 6, 7, 8],
    [1, 2, 3, 4],
    [1, 2, 3, 4],
    [1, 2, 3, 4, 5, 6, 7, 8],
    [1, 2, 3, 3, 4, 4, 5, 5],
    [1, 2, 3, 3, 4, 4, 5, 5],
    [1, 2, 3, 3, 4, 4, 5, 5],
    [1, 2, 3, 3, 4, 4, 5, 5, 6, 7, 8, 8, 9, 9, 10, 10],
    [1, 2, 3],
    [1, 2, 3, 4, 5, 6],
    [1, 2, 2, 3, 3, 3],
    [1, 2, 2, 3, 3, 3],
    [1, 2, 2, 3, 3, 3, 4, 5, 5, 6, 6, 6],
    [1, 2, 3, 4, 5, 6],
    [1, 2, 3, 4, 5, 6],
    [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12],
    [1, 2, 2, 3, 4, 4, 5, 5, 5, 6, 6, 6],
    [1, 2, 2, 3, 4, 4, 5, 5, 5, 6, 6, 6],
    [1, 2, 2, 3, 4, 4, 5, 5, 5, 6, 6, 6],
    [1, 2, 2, 3, 4, 4, 5, 5, 5, 6, 6, 6, 7, 8, 8, 9, 10, 10, 11, 11, 11, 12, 12, 12],
    [1, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4],
    [1, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 5, 6, 6, 6, 7, 7, 7, 7, 8, 8, 8, 8],
    [1, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 5, 5, 5, 4, 4, 5, 5, 4, 5, 4],
    [1, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 5, 5, 4, 5, 5, 4, 4, 5, 4, 5],
    [1, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 5, 5, 5, 4, 4, 5, 5, 4, 5, 4, 6, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 9, 9, 10, 10, 10, 9, 9, 10, 10, 9, 10, 9]
]

const OperationNames = [
    ["1"],
    ["1", "-1"],
    ["1", "2₀₁₀"],
    ["1", "m₀₁₀"],
    ["1", "2₀₁₀", "-1", "m₀₁₀"],
    ["1", "2₀₀₁", "2₀₁₀", "2₁₀₀"],
    ["1", "2₀₀₁", "m₀₁₀", "m₁₀₀"],
    ["1", "2₀₀₁", "2₀₁₀", "2₁₀₀", "-1", "m₀₀₁", "m₀₁₀", "m₁₀₀"],
    ["1", "2₀₀₁", "4₀₀₁⁺", "4₀₀₁⁻"],
    ["1", "2₀₀₁", "-4₀₀₁⁺", "-4₀₀₁⁻"],
    ["1", "2₀₀₁", "4₀₀₁⁺", "4₀₀₁⁻", "-1", "m₀₀₁", "-4₀₀₁⁺", "-4₀₀₁⁻"],
    ["1", "2₀₀₁", "4₀₀₁⁺", "4₀₀₁⁻", "2₀₁₀", "2₁₀₀", "2₁₁₀", "2₋₁₁₀"],
    ["1", "2₀₀₁", "4₀₀₁⁺", "4₀₀₁⁻", "m₀₁₀", "m₁₀₀", "m₁₁₀", "m₋₁₁₀"],
    ["1", "2₀₀₁", "-4₀₀₁⁺", "-4₀₀₁⁻", "2₀₁₀", "2₁₀₀", "m₁₁₀", "m₋₁₁₀"],
    ["1", "2₀₀₁", "4₀₀₁⁺", "4₀₀₁⁻", "2₀₁₀", "2₁₀₀", "2₁₁₀", "2₋₁₁₀", "-1", "m₀₀₁", "-4₀₀₁⁺", "-4₀₀₁⁻", "m₀₁₀", "m₁₀₀", "m₁₁₀", "m₋₁₁₀"],
    ["1", "3₀₀₁⁺", "3₀₀₁⁻"],
    ["1", "3₀₀₁⁺", "3₀₀₁⁻", "-1", "-3₀₀₁⁺", "-3₀₀₁⁻"],
    ["1", "3₀₀₁⁺", "3₀₀₁⁻", "2₋₁₁₀", "2₁₂₀", "2₂₁₀"],
    ["1", "3₀₀₁⁺", "3₀₀₁⁻", "m₁₁₀", "m₁₀₀", "m₀₁₀"],
    ["1", "3₀₀₁⁺", "3₀₀₁⁻", "2₋₁₁₀", "2₁₂₀", "2₂₁₀", "-1", "-3₀₀₁⁺", "-3₀₀₁⁻", "m₋₁₁₀", "m₁₂₀", "m₂₁₀"],
    ["1", "3₀₀₁⁺", "3₀₀₁⁻", "2₀₀₁", "6₀₀₁⁻", "6₀₀₁⁺"],
    ["1", "3₀₀₁⁺", "3₀₀₁⁻", "m₀₀₁", "-6₀₀₁⁻", "-6₀₀₁⁺"],
    ["1", "3₀₀₁⁺", "3₀₀₁⁻", "2₀₀₁", "6₀₀₁⁻", "6₀₀₁⁺", "-1", "-3₀₀₁⁺", "-3₀₀₁⁻", "m₀₀₁", "-6₀₀₁⁻", "-6₀₀₁⁺"],
    ["1", "3₀₀₁⁺", "3₀₀₁⁻", "2₀₀₁", "6₀₀₁⁻", "6₀₀₁⁺", "2₁₁₀", "2₁₀₀", "2₀₁₀", "2₋₁₁₀", "2₁₂₀", "2₂₁₀"],
    ["1", "3₀₀₁⁺", "3₀₀₁⁻", "2₀₀₁", "6₀₀₁⁻", "6₀₀₁⁺", "m₁₁₀", "m₁₀₀", "m₀₁₀", "m₋₁₁₀", "m₁₂₀", "m₂₁₀"],
    ["1", "3₀₀₁⁺", "3₀₀₁⁻", "m₀₀₁", "-6₀₀₁⁻", "-6₀₀₁⁺", "2₁₁₀", "2₁₀₀", "2₀₁₀", "m₋₁₁₀", "m₁₂₀", "m₂₁₀"],
    ["1", "3₀₀₁⁺", "3₀₀₁⁻", "2₀₀₁", "6₀₀₁⁻", "6₀₀₁⁺", "2₁₁₀", "2₁₀₀", "2₀₁₀", "2₋₁₁₀", "2₁₂₀", "2₂₁₀", "-1", "-3₀₀₁⁺", "-3₀₀₁⁻", "m₀₀₁", "-6₀₀₁⁻", "-6₀₀₁⁺", "m₁₁₀", "m₁₀₀", "m₀₁₀", "m₋₁₁₀", "m₁₂₀", "m₂₁₀"],
    ["1", "2₀₀₁", "2₀₁₀", "2₁₀₀", "3₁₁₁⁺", "3₋₁₁₋₁⁺", "3₋₁₁₁⁻", "3₋₁₋₁₁⁺", "3₁₁₁⁻", "3₋₁₁₁⁺", "3₋₁₋₁₁⁻", "3₋₁₁₋₁⁻"],
    ["1", "2₀₀₁", "2₀₁₀", "2₁₀₀", "3₁₁₁⁺", "3₋₁₁₋₁⁺", "3₋₁₁₁⁻", "3₋₁₋₁₁⁺", "3₁₁₁⁻", "3₋₁₁₁⁺", "3₋₁₋₁₁⁻", "3₋₁₁₋₁⁻", "-1", "m₀₀₁", "m₀₁₀", "m₁₀₀", "-3₁₁₁⁺", "-3₋₁₁₋₁⁺", "-3₋₁₁₁⁻", "-3₋₁₋₁₁⁺", "-3₁₁₁⁻", "-3₋₁₁₁⁺", "-3₋₁₋₁₁⁻", "-3₋₁₁₋₁⁻"],
    ["1", "2₀₀₁", "2₀₁₀", "2₁₀₀", "3₁₁₁⁺", "3₋₁₁₋₁⁺", "3₋₁₁₁⁻", "3₋₁₋₁₁⁺", "3₁₁₁⁻", "3₋₁₁₁⁺", "3₋₁₋₁₁⁻", "3₋₁₁₋₁⁻", "2₁₁₀", "2₋₁₁₀", "4₀₀₁⁻", "4₀₀₁⁺", "4₁₀₀⁻", "2₀₁₁", "2₀₋₁₁", "4₁₀₀⁺", "4₀₁₀⁺", "2₁₀₁", "4₀₁₀⁻", "2₋₁₀₁"],
    ["1", "2₀₀₁", "2₀₁₀", "2₁₀₀", "3₁₁₁⁺", "3₋₁₁₋₁⁺", "3₋₁₁₁⁻", "3₋₁₋₁₁⁺", "3₁₁₁⁻", "3₋₁₁₁⁺", "3₋₁₋₁₁⁻", "3₋₁₁₋₁⁻", "m₋₁₁₀", "m₁₁₀", "-4₀₀₁⁺", "-4₀₀₁⁻", "m₀₋₁₁", "-4₁₀₀⁺", "-4₁₀₀⁻", "m₀₁₁", "m₋₁₀₁", "-4₀₁₀⁻", "m₁₀₁", "-4₀₁₀⁺"],
    ["1", "2₀₀₁", "2₀₁₀", "2₁₀₀", "3₁₁₁⁺", "3₋₁₁₋₁⁺", "3₋₁₁₁⁻", "3₋₁₋₁₁⁺", "3₁₁₁⁻", "3₋₁₁₁⁺", "3₋₁₋₁₁⁻", "3₋₁₁₋₁⁻", "2₁₁₀", "2₋₁₁₀", "4₀₀₁⁻", "4₀₀₁⁺", "4₁₀₀⁻", "2₀₁₁", "2₀₋₁₁", "4₁₀₀⁺", "4₀₁₀⁺", "2₁₀₁", "4₀₁₀⁻", "2₋₁₀₁", "-1", "m₀₀₁", "m₀₁₀", "m₁₀₀", "-3₁₁₁⁺", "-3₋₁₁₋₁⁺", "-3₋₁₁₁⁻", "-3₋₁₋₁₁⁺", "-3₁₁₁⁻", "-3₋₁₁₁⁺", "-3₋₁₋₁₁⁻", "-3₋₁₁₋₁⁻", "m₁₁₀", "m₋₁₁₀", "-4₀₀₁⁻", "-4₀₀₁⁺", "-4₁₀₀⁻", "m₀₁₁", "m₀₋₁₁", "-4₁₀₀⁺", "-4₀₁₀⁺", "m₁₀₁", "-4₀₁₀⁻", "m₋₁₀₁"]
]

const HexagonalAxes = [1.0 -0.5 0.0; 0.0 sqrt(3)/2 0.0; 0.0 0.0 1.0]
const HexagonalGroups = Bool[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0]
const RotationMatrices = [
    [[1 0 0; 0 1 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 -1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 -1]],
    [[1 0 0; 0 1 0; 0 0 1], [1 0 0; 0 -1 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 -1], [-1 0 0; 0 -1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 -1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 -1], [-1 0 0; 0 -1 0; 0 0 -1], [1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [0 -1 0; 1 0 0; 0 0 1], [0 1 0; -1 0 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [0 1 0; -1 0 0; 0 0 -1], [0 -1 0; 1 0 0; 0 0 -1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [0 -1 0; 1 0 0; 0 0 1], [0 1 0; -1 0 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 -1], [1 0 0; 0 1 0; 0 0 -1], [0 1 0; -1 0 0; 0 0 -1], [0 -1 0; 1 0 0; 0 0 -1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [0 -1 0; 1 0 0; 0 0 1], [0 1 0; -1 0 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 -1], [0 1 0; 1 0 0; 0 0 -1], [0 -1 0; -1 0 0; 0 0 -1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [0 -1 0; 1 0 0; 0 0 1], [0 1 0; -1 0 0; 0 0 1], [1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 1], [0 -1 0; -1 0 0; 0 0 1], [0 1 0; 1 0 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [0 1 0; -1 0 0; 0 0 -1], [0 -1 0; 1 0 0; 0 0 -1], [-1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 -1], [0 -1 0; -1 0 0; 0 0 1], [0 1 0; 1 0 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [0 -1 0; 1 0 0; 0 0 1], [0 1 0; -1 0 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 -1], [0 1 0; 1 0 0; 0 0 -1], [0 -1 0; -1 0 0; 0 0 -1], [-1 0 0; 0 -1 0; 0 0 -1], [1 0 0; 0 1 0; 0 0 -1], [0 1 0; -1 0 0; 0 0 -1], [0 -1 0; 1 0 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 1], [0 -1 0; -1 0 0; 0 0 1], [0 1 0; 1 0 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [0 -1 0; 1 -1 0; 0 0 1], [-1 1 0; -1 0 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [0 -1 0; 1 -1 0; 0 0 1], [-1 1 0; -1 0 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 -1], [0 1 0; -1 1 0; 0 0 -1], [1 -1 0; 1 0 0; 0 0 -1]],
    [[1 0 0; 0 1 0; 0 0 1], [0 -1 0; 1 -1 0; 0 0 1], [-1 1 0; -1 0 0; 0 0 1], [0 -1 0; -1 0 0; 0 0 -1], [-1 1 0; 0 1 0; 0 0 -1], [1 0 0; 1 -1 0; 0 0 -1]],
    [[1 0 0; 0 1 0; 0 0 1], [0 -1 0; 1 -1 0; 0 0 1], [-1 1 0; -1 0 0; 0 0 1], [0 -1 0; -1 0 0; 0 0 1], [-1 1 0; 0 1 0; 0 0 1], [1 0 0; 1 -1 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [0 -1 0; 1 -1 0; 0 0 1], [-1 1 0; -1 0 0; 0 0 1], [0 -1 0; -1 0 0; 0 0 -1], [-1 1 0; 0 1 0; 0 0 -1], [1 0 0; 1 -1 0; 0 0 -1], [-1 0 0; 0 -1 0; 0 0 -1], [0 1 0; -1 1 0; 0 0 -1], [1 -1 0; 1 0 0; 0 0 -1], [0 1 0; 1 0 0; 0 0 1], [1 -1 0; 0 -1 0; 0 0 1], [-1 0 0; -1 1 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [0 -1 0; 1 -1 0; 0 0 1], [-1 1 0; -1 0 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [0 1 0; -1 1 0; 0 0 1], [1 -1 0; 1 0 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [0 -1 0; 1 -1 0; 0 0 1], [-1 1 0; -1 0 0; 0 0 1], [1 0 0; 0 1 0; 0 0 -1], [0 -1 0; 1 -1 0; 0 0 -1], [-1 1 0; -1 0 0; 0 0 -1]],
    [[1 0 0; 0 1 0; 0 0 1], [0 -1 0; 1 -1 0; 0 0 1], [-1 1 0; -1 0 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [0 1 0; -1 1 0; 0 0 1], [1 -1 0; 1 0 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 -1], [0 1 0; -1 1 0; 0 0 -1], [1 -1 0; 1 0 0; 0 0 -1], [1 0 0; 0 1 0; 0 0 -1], [0 -1 0; 1 -1 0; 0 0 -1], [-1 1 0; -1 0 0; 0 0 -1]],
    [[1 0 0; 0 1 0; 0 0 1], [0 -1 0; 1 -1 0; 0 0 1], [-1 1 0; -1 0 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [0 1 0; -1 1 0; 0 0 1], [1 -1 0; 1 0 0; 0 0 1], [0 1 0; 1 0 0; 0 0 -1], [1 -1 0; 0 -1 0; 0 0 -1], [-1 0 0; -1 1 0; 0 0 -1], [0 -1 0; -1 0 0; 0 0 -1], [-1 1 0; 0 1 0; 0 0 -1], [1 0 0; 1 -1 0; 0 0 -1]],
    [[1 0 0; 0 1 0; 0 0 1], [0 -1 0; 1 -1 0; 0 0 1], [-1 1 0; -1 0 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [0 1 0; -1 1 0; 0 0 1], [1 -1 0; 1 0 0; 0 0 1], [0 -1 0; -1 0 0; 0 0 1], [-1 1 0; 0 1 0; 0 0 1], [1 0 0; 1 -1 0; 0 0 1], [0 1 0; 1 0 0; 0 0 1], [1 -1 0; 0 -1 0; 0 0 1], [-1 0 0; -1 1 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [0 -1 0; 1 -1 0; 0 0 1], [-1 1 0; -1 0 0; 0 0 1], [1 0 0; 0 1 0; 0 0 -1], [0 -1 0; 1 -1 0; 0 0 -1], [-1 1 0; -1 0 0; 0 0 -1], [0 1 0; 1 0 0; 0 0 -1], [1 -1 0; 0 -1 0; 0 0 -1], [-1 0 0; -1 1 0; 0 0 -1], [0 1 0; 1 0 0; 0 0 1], [1 -1 0; 0 -1 0; 0 0 1], [-1 0 0; -1 1 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [0 -1 0; 1 -1 0; 0 0 1], [-1 1 0; -1 0 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [0 1 0; -1 1 0; 0 0 1], [1 -1 0; 1 0 0; 0 0 1], [0 1 0; 1 0 0; 0 0 -1], [1 -1 0; 0 -1 0; 0 0 -1], [-1 0 0; -1 1 0; 0 0 -1], [0 -1 0; -1 0 0; 0 0 -1], [-1 1 0; 0 1 0; 0 0 -1], [1 0 0; 1 -1 0; 0 0 -1], [-1 0 0; 0 -1 0; 0 0 -1], [0 1 0; -1 1 0; 0 0 -1], [1 -1 0; 1 0 0; 0 0 -1], [1 0 0; 0 1 0; 0 0 -1], [0 -1 0; 1 -1 0; 0 0 -1], [-1 1 0; -1 0 0; 0 0 -1], [0 -1 0; -1 0 0; 0 0 1], [-1 1 0; 0 1 0; 0 0 1], [1 0 0; 1 -1 0; 0 0 1], [0 1 0; 1 0 0; 0 0 1], [1 -1 0; 0 -1 0; 0 0 1], [-1 0 0; -1 1 0; 0 0 1]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 -1], [0 0 1; 1 0 0; 0 1 0], [0 0 1; -1 0 0; 0 -1 0], [0 0 -1; -1 0 0; 0 1 0], [0 0 -1; 1 0 0; 0 -1 0], [0 1 0; 0 0 1; 1 0 0], [0 -1 0; 0 0 1; -1 0 0], [0 1 0; 0 0 -1; -1 0 0], [0 -1 0; 0 0 -1; 1 0 0]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 -1], [0 0 1; 1 0 0; 0 1 0], [0 0 1; -1 0 0; 0 -1 0], [0 0 -1; -1 0 0; 0 1 0], [0 0 -1; 1 0 0; 0 -1 0], [0 1 0; 0 0 1; 1 0 0], [0 -1 0; 0 0 1; -1 0 0], [0 1 0; 0 0 -1; -1 0 0], [0 -1 0; 0 0 -1; 1 0 0], [-1 0 0; 0 -1 0; 0 0 -1], [1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 1], [0 0 -1; -1 0 0; 0 -1 0], [0 0 -1; 1 0 0; 0 1 0], [0 0 1; 1 0 0; 0 -1 0], [0 0 1; -1 0 0; 0 1 0], [0 -1 0; 0 0 -1; -1 0 0], [0 1 0; 0 0 -1; 1 0 0], [0 -1 0; 0 0 1; 1 0 0], [0 1 0; 0 0 1; -1 0 0]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 -1], [0 0 1; 1 0 0; 0 1 0], [0 0 1; -1 0 0; 0 -1 0], [0 0 -1; -1 0 0; 0 1 0], [0 0 -1; 1 0 0; 0 -1 0], [0 1 0; 0 0 1; 1 0 0], [0 -1 0; 0 0 1; -1 0 0], [0 1 0; 0 0 -1; -1 0 0], [0 -1 0; 0 0 -1; 1 0 0], [0 1 0; 1 0 0; 0 0 -1], [0 -1 0; -1 0 0; 0 0 -1], [0 1 0; -1 0 0; 0 0 1], [0 -1 0; 1 0 0; 0 0 1], [1 0 0; 0 0 1; 0 -1 0], [-1 0 0; 0 0 1; 0 1 0], [-1 0 0; 0 0 -1; 0 -1 0], [1 0 0; 0 0 -1; 0 1 0], [0 0 1; 0 1 0; -1 0 0], [0 0 1; 0 -1 0; 1 0 0], [0 0 -1; 0 1 0; 1 0 0], [0 0 -1; 0 -1 0; -1 0 0]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 -1], [0 0 1; 1 0 0; 0 1 0], [0 0 1; -1 0 0; 0 -1 0], [0 0 -1; -1 0 0; 0 1 0], [0 0 -1; 1 0 0; 0 -1 0], [0 1 0; 0 0 1; 1 0 0], [0 -1 0; 0 0 1; -1 0 0], [0 1 0; 0 0 -1; -1 0 0], [0 -1 0; 0 0 -1; 1 0 0], [0 1 0; 1 0 0; 0 0 1], [0 -1 0; -1 0 0; 0 0 1], [0 1 0; -1 0 0; 0 0 -1], [0 -1 0; 1 0 0; 0 0 -1], [1 0 0; 0 0 1; 0 1 0], [-1 0 0; 0 0 1; 0 -1 0], [-1 0 0; 0 0 -1; 0 1 0], [1 0 0; 0 0 -1; 0 -1 0], [0 0 1; 0 1 0; 1 0 0], [0 0 1; 0 -1 0; -1 0 0], [0 0 -1; 0 1 0; -1 0 0], [0 0 -1; 0 -1 0; 1 0 0]],
    [[1 0 0; 0 1 0; 0 0 1], [-1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 -1], [0 0 1; 1 0 0; 0 1 0], [0 0 1; -1 0 0; 0 -1 0], [0 0 -1; -1 0 0; 0 1 0], [0 0 -1; 1 0 0; 0 -1 0], [0 1 0; 0 0 1; 1 0 0], [0 -1 0; 0 0 1; -1 0 0], [0 1 0; 0 0 -1; -1 0 0], [0 -1 0; 0 0 -1; 1 0 0], [0 1 0; 1 0 0; 0 0 -1], [0 -1 0; -1 0 0; 0 0 -1], [0 1 0; -1 0 0; 0 0 1], [0 -1 0; 1 0 0; 0 0 1], [1 0 0; 0 0 1; 0 -1 0], [-1 0 0; 0 0 1; 0 1 0], [-1 0 0; 0 0 -1; 0 -1 0], [1 0 0; 0 0 -1; 0 1 0], [0 0 1; 0 1 0; -1 0 0], [0 0 1; 0 -1 0; 1 0 0], [0 0 -1; 0 1 0; 1 0 0], [0 0 -1; 0 -1 0; -1 0 0], [-1 0 0; 0 -1 0; 0 0 -1], [1 0 0; 0 1 0; 0 0 -1], [1 0 0; 0 -1 0; 0 0 1], [-1 0 0; 0 1 0; 0 0 1], [0 0 -1; -1 0 0; 0 -1 0], [0 0 -1; 1 0 0; 0 1 0], [0 0 1; 1 0 0; 0 -1 0], [0 0 1; -1 0 0; 0 1 0], [0 -1 0; 0 0 -1; -1 0 0], [0 1 0; 0 0 -1; 1 0 0], [0 -1 0; 0 0 1; 1 0 0], [0 1 0; 0 0 1; -1 0 0], [0 -1 0; -1 0 0; 0 0 1], [0 1 0; 1 0 0; 0 0 1], [0 -1 0; 1 0 0; 0 0 -1], [0 1 0; -1 0 0; 0 0 -1], [-1 0 0; 0 0 -1; 0 1 0], [1 0 0; 0 0 -1; 0 -1 0], [1 0 0; 0 0 1; 0 1 0], [-1 0 0; 0 0 1; 0 -1 0], [0 0 -1; 0 -1 0; 1 0 0], [0 0 -1; 0 1 0; -1 0 0], [0 0 1; 0 -1 0; -1 0 0], [0 0 1; 0 1 0; 1 0 0]]
]

const RepresentationNames = [
    ["Γ₁"],
    ["Γ₁⁺", "Γ₁⁻"],
    ["Γ₁", "Γ₂"],
    ["Γ₁", "Γ₂"],
    ["Γ₁⁺", "Γ₁⁻", "Γ₂⁺", "Γ₂⁻"],
    ["Γ₁", "Γ₂", "Γ₄", "Γ₃"],
    ["Γ₁", "Γ₂", "Γ₄", "Γ₃"],
    ["Γ₁⁺", "Γ₁⁻", "Γ₂⁺", "Γ₂⁻", "Γ₄⁺", "Γ₄⁻", "Γ₃⁺", "Γ₃⁻"],
    ["Γ₁", "Γ₂", "Γ₃", "Γ₄"],
    ["Γ₁", "Γ₂", "Γ₃", "Γ₄"],
    ["Γ₁⁺", "Γ₁⁻", "Γ₂⁺", "Γ₂⁻", "Γ₃⁺", "Γ₃⁻", "Γ₄⁺", "Γ₄⁻"],
    ["Γ₁", "Γ₃", "Γ₂", "Γ₄", "Γ₅"],
    ["Γ₁", "Γ₄", "Γ₂", "Γ₃", "Γ₅"],
    ["Γ₁", "Γ₄", "Γ₂", "Γ₃", "Γ₅"],
    ["Γ₁⁺", "Γ₁⁻", "Γ₃⁺", "Γ₃⁻", "Γ₂⁺", "Γ₂⁻", "Γ₄⁺", "Γ₄⁻", "Γ₅⁺", "Γ₅⁻"],
    ["Γ₁", "Γ₂", "Γ₃"],
    ["Γ₁⁺", "Γ₁⁻", "Γ₂⁺", "Γ₂⁻", "Γ₃⁺", "Γ₃⁻"],
    ["Γ₁", "Γ₂", "Γ₃"],
    ["Γ₁", "Γ₂", "Γ₃"],
    ["Γ₁⁺", "Γ₁⁻", "Γ₂⁺", "Γ₂⁻", "Γ₃⁺", "Γ₃⁻"],
    ["Γ₁", "Γ₂", "Γ₃", "Γ₄", "Γ₅", "Γ₆"],
    ["Γ₁", "Γ₂", "Γ₃", "Γ₄", "Γ₅", "Γ₆"],
    ["Γ₁⁺", "Γ₁⁻", "Γ₂⁺", "Γ₂⁻", "Γ₃⁺", "Γ₃⁻", "Γ₄⁺", "Γ₄⁻", "Γ₅⁺", "Γ₅⁻", "Γ₆⁺", "Γ₆⁻"],
    ["Γ₁", "Γ₂", "Γ₃", "Γ₄", "Γ₅", "Γ₆"],
    ["Γ₁", "Γ₂", "Γ₄", "Γ₃", "Γ₅", "Γ₆"],
    ["Γ₁", "Γ₄", "Γ₂", "Γ₃", "Γ₅", "Γ₆"],
    ["Γ₁⁺", "Γ₁⁻", "Γ₂⁺", "Γ₂⁻", "Γ₃⁺", "Γ₃⁻", "Γ₄⁺", "Γ₄⁻", "Γ₅⁺", "Γ₅⁻", "Γ₆⁺", "Γ₆⁻"],
    ["Γ₁", "Γ₂", "Γ₃", "Γ₄"],
    ["Γ₁⁺", "Γ₁⁻", "Γ₂⁺", "Γ₂⁻", "Γ₃⁺", "Γ₃⁻", "Γ₄⁺", "Γ₄⁻"],
    ["Γ₁", "Γ₂", "Γ₃", "Γ₅", "Γ₄"],
    ["Γ₁", "Γ₂", "Γ₃", "Γ₄", "Γ₅"],
    ["Γ₁⁺", "Γ₁⁻", "Γ₂⁺", "Γ₂⁻", "Γ₃⁺", "Γ₃⁻", "Γ₅⁺", "Γ₅⁻", "Γ₄⁺", "Γ₄⁻"]
]

const RepresentationNames2 = [
    ["GM1"],
    ["GM1+", "GM1-"],
    ["GM1", "GM2"],
    ["GM1", "GM2"],
    ["GM1+", "GM1-", "GM2+", "GM2-"],
    ["GM1", "GM2", "GM4", "GM3"],
    ["GM1", "GM2", "GM4", "GM3"],
    ["GM1+", "GM1-", "GM2+", "GM2-", "GM4+", "GM4-", "GM3+", "GM3-"],
    ["GM1", "GM2", "GM3", "GM4"],
    ["GM1", "GM2", "GM3", "GM4"],
    ["GM1+", "GM1-", "GM2+", "GM2-", "GM3+", "GM3-", "GM4+", "GM4-"],
    ["GM1", "GM3", "GM2", "GM4", "GM5"],
    ["GM1", "GM4", "GM2", "GM3", "GM5"],
    ["GM1", "GM4", "GM2", "GM3", "GM5"],
    ["GM1+", "GM1-", "GM3+", "GM3-", "GM2+", "GM2-", "GM4+", "GM4-", "GM5+", "GM5-"],
    ["GM1", "GM2", "GM3"],
    ["GM1+", "GM1-", "GM2+", "GM2-", "GM3+", "GM3-"],
    ["GM1", "GM2", "GM3"],
    ["GM1", "GM2", "GM3"],
    ["GM1+", "GM1-", "GM2+", "GM2-", "GM3+", "GM3-"],
    ["GM1", "GM2", "GM3", "GM4", "GM5", "GM6"],
    ["GM1", "GM2", "GM3", "GM4", "GM5", "GM6"],
    ["GM1+", "GM1-", "GM2+", "GM2-", "GM3+", "GM3-", "GM4+", "GM4-", "GM5+", "GM5-", "GM6+", "GM6-"],
    ["GM1", "GM2", "GM3", "GM4", "GM5", "GM6"],
    ["GM1", "GM2", "GM4", "GM3", "GM5", "GM6"],
    ["GM1", "GM4", "GM2", "GM3", "GM5", "GM6"],
    ["GM1+", "GM1-", "GM2+", "GM2-", "GM3+", "GM3-", "GM4+", "GM4-", "GM5+", "GM5-", "GM6+", "GM6-"],
    ["GM1", "GM2", "GM3", "GM4"],
    ["GM1+", "GM1-", "GM2+", "GM2-", "GM3+", "GM3-", "GM4+", "GM4-"],
    ["GM1", "GM2", "GM3", "GM5", "GM4"],
    ["GM1", "GM2", "GM3", "GM4", "GM5"],
    ["GM1+", "GM1-", "GM2+", "GM2-", "GM3+", "GM3-", "GM5+", "GM5-", "GM4+", "GM4-"]
]

const RepresentationNames3 = [
    ["A"],
    ["Ag", "Au"],
    ["A", "B"],
    ["A'", "A''"],
    ["Ag", "Au", "Bg", "Bu"],
    ["A", "B2", "B3", "B1"],
    ["A", "B1", "B2", "A2"],
    ["Ag", "Au", "B2g", "B2u", "B3g", "B3u", "B1g", "B1u"],
    ["A", "B", "2E", "1E"],
    ["A", "B", "2E", "1E"],
    ["Ag", "Au", "Bg", "Bu", "2Eg", "2Eu", "1Eg", "1Eu"],
    ["A1", "B1", "A2", "B2", "E"],
    ["A1", "B2", "A2", "B1", "E"],
    ["A1", "B2", "A2", "B1", "E"],
    ["A1g", "A1u", "B1g", "B1u", "A2g", "A2u", "B2g", "B2u", "Eg", "Eu"],
    ["A1", "2E", "1E"],
    ["A1g", "A1u", "2Eg", "2Eu", "1Eg", "1Eu"],
    ["A1", "A2", "E"],
    ["A1", "A2", "E"],
    ["A1g", "A1u", "A2g", "A2u", "Eg", "Eu"],
    ["A", "2E2", "1E2", "B", "2E1", "1E1"],
    ["A'", "2E'", "1E'", "A''", "2E''", "1E''"],
    ["Ag", "Au", "2E2g", "2E2u", "1E2g", "1E2u", "Bg", "Bu", "2E1g", "2E1u", "1E1g", "1E1u"],
    ["A1", "A2", "B1", "B2", "E1", "E2"],
    ["A1", "A2", "B1", "B2", "E1", "E2"],
    ["A1'", "A2''", "A2'", "A1''", "E''", "E'"],
    ["A1g", "A1u", "A2g", "A2u", "B1g", "B1u", "B2g", "B2u", "E1g", "E1u", "E2g", "E2u"],
    ["A", "1E", "2E", "T"],
    ["Ag", "1Au", "1Eg", "1Eu", "2Eg", "2Eu", "Tg", "Tu"],
    ["A1", "A2", "E", "T2", "T1"],
    ["A1", "A2", "E", "T1", "T2"],
    ["A1g", "A1u", "A2g", "A2u", "Eg", "Eu", "T2g", "T2u", "T1g", "T1u"]
]